﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;

namespace Support_128338
{
    public delegate void ItemInCollectionChangeEventHandler(object sender, ItemInCollectionChangeEventArgs e);

    public class ItemsChangeObservableCollection<T> : ObservableCollection<T>
    {
        public event ItemInCollectionChangeEventHandler ItemInCollectionChangeEvent;

        public ItemsChangeObservableCollection(IEnumerable<T> items)
        {
            foreach (var item in items)
            {
                Add(item);
            }
        }

        protected virtual void OnItemInCollectionChange(ItemInCollectionChangeEventArgs e)
        {
            ItemInCollectionChangeEventHandler handler = ItemInCollectionChangeEvent;
            handler?.Invoke(this, e);
        }

        protected override void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                RegisterPropertyChanged(e.NewItems);
            }
            else if (e.Action == NotifyCollectionChangedAction.Remove)
            {
                UnRegisterPropertyChanged(e.OldItems);
            }
            else if (e.Action == NotifyCollectionChangedAction.Replace)
            {
                UnRegisterPropertyChanged(e.OldItems);
                RegisterPropertyChanged(e.NewItems);
            }
            base.OnCollectionChanged(e);
        }

        protected override void ClearItems()
        {
            UnRegisterPropertyChanged(this);
            base.ClearItems();
        }

        private void RegisterPropertyChanged(IEnumerable items)
        {
            foreach (INotifyPropertyChanged item in items)
            {
                if (item != null)
                {
                    item.PropertyChanged += item_PropertyChanged;
                }
            }
        }

        private void UnRegisterPropertyChanged(IEnumerable items)
        {
            foreach (INotifyPropertyChanged item in items)
            {
                if (item != null)
                {
                    item.PropertyChanged -= item_PropertyChanged;
                }
            }
        }

        private void item_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            OnItemInCollectionChange(new ItemInCollectionChangeEventArgs { Item = sender, PropertyChangedEventArgs = e });
            //base.OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
        }
    }


    public class ItemInCollectionChangeEventArgs : EventArgs
    {
        public object Item { get; internal set; }
        public PropertyChangedEventArgs PropertyChangedEventArgs { get; internal set; }
    }

}
