﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using Syncfusion.UI.Xaml.Grid;
using Syncfusion.UI.Xaml.Grid.Cells;
using Syncfusion.Windows.Shared;

namespace Support_128338
{
    public partial class MainWindow
    {
        public MainWindow()
        {
            InitializeComponent();
            Sfdatagrid.CellRenderers.Remove("Numeric");
            Sfdatagrid.CellRenderers.Add("Numeric", new GridNumericColumnRendererExt());
        }
    }

    public class DecimalDigitConverter : IValueConverter
    {
        object IValueConverter.Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            var model = value as Model;
            if (model == null) return null;

            var decimalDegits = model.NumberOfDecimals;
            return decimalDegits == 0 ? Math.Floor(model.Value) : Math.Round(model.Value, 2);
        }
        object IValueConverter.ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return value;
        }
    }

    public class GridNumericColumnRendererExt : GridCellNumericRenderer
    {
        public override void OnInitializeEditElement(DataColumnBase dataColumn, DoubleTextBox uiElement, object dataContext)
        {
            base.OnInitializeEditElement(dataColumn, uiElement, dataContext);
            if (dataColumn.GridColumn.MappingName == "Value")
            {
                var model = dataContext as Model;
                if (model == null) return;

                int decimalDigits = model.NumberOfDecimals;
                uiElement.NumberDecimalDigits = decimalDigits;
                uiElement.Value = Math.Round(model.Value, decimalDigits);
                if (decimalDigits == 0)
                {
                    uiElement.Value = Math.Floor(model.Value);
                    uiElement.NumberDecimalDigits = 0;
                }
                else
                {
                    uiElement.Value = Math.Round(model.Value, 2);
                    uiElement.NumberDecimalDigits = 2;
                }
                var column = dataColumn.GridColumn;
                var textPadding = new Binding { Path = new PropertyPath("Padding"), Mode = BindingMode.OneWay, Source = column };
                uiElement.SetBinding(Control.PaddingProperty, textPadding);
                var textAlignBind = new Binding { Path = new PropertyPath("TextAlignment"), Mode = BindingMode.TwoWay, Source = column };
                uiElement.SetBinding(TextBox.TextAlignmentProperty, textAlignBind);
                var textWrappingBinding = new Binding { Path = new PropertyPath("TextWrapping"), Mode = BindingMode.TwoWay, Source = column };
                uiElement.SetBinding(TextBox.TextWrappingProperty, textWrappingBinding);
                var verticalContentAlignment = new Binding { Path = new PropertyPath("VerticalAlignment"), Mode = BindingMode.TwoWay, Source = column };
                uiElement.SetBinding(Control.VerticalContentAlignmentProperty, verticalContentAlignment);
                var textDecorations = new Binding { Path = new PropertyPath("TextDecorations"), Mode = BindingMode.TwoWay, Source = column };
                uiElement.SetBinding(TextBox.TextDecorationsProperty, textDecorations);
            }
        }
    }

    public class SfDataGridExt : SfDataGrid
    {
        protected override void OnTextInput(TextCompositionEventArgs e)
        {
            if (!SelectionController.CurrentCellManager.HasCurrentCell)
            {
                base.OnTextInput(e);
                return;
            }

            //Get the Current Row and Column index from the CurrentCellManager
            var rowColumnIndex = SelectionController.CurrentCellManager.CurrentRowColumnIndex;
            var rowGenerator = RowGenerator;
            //Get the row from the Row index
            var dataRow = rowGenerator.Items.FirstOrDefault(item => item.RowIndex == rowColumnIndex.RowIndex);
            //Check whether the datarow is null or not and the type as DataRow
            if (dataRow is DataRow)
            {
                //Get the column from the VisibleColumn collection based on the column index
                var dataColumn = dataRow.VisibleColumns.FirstOrDefault(column => column.ColumnIndex == rowColumnIndex.ColumnIndex);
                //Convert the input text to char type 
                char text;
                char.TryParse(e.Text, out text);
                //Skip if the column is GridTemplateColumn and the column is not already in editing 
                //Allow Editing only pressed letters digits and Minus sign key 
                if (dataColumn != null
                    && !(dataColumn.GridColumn is GridTemplateColumn)
                    && !dataColumn.IsEditing
                    && SelectionController.CurrentCellManager.BeginEdit()
                    && (e.Text.Equals("-") || char.IsLetterOrDigit(text)))
                    dataColumn.Renderer.PreviewTextInput(e);
            }
            base.OnTextInput(e);
        }
    }
}
