﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;

namespace Support_128338
{
    class ViewModel : INotifyPropertyChanged
    {
        private ItemsChangeObservableCollection<Model> _valueList;
        public ItemsChangeObservableCollection<Model> ValueList
        {
            get { return _valueList; }
            set
            {
                _valueList = value;
                OnPropertyChanged("ValueList");
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        public ViewModel()
        {
            var valueList = new List<Model>
            {
                new Model {ItemName = "Item 1", ItemType = "A", NumberOfDecimals = 2, Value = 13.1},
                new Model {ItemName = "Item 2", ItemType = "A", NumberOfDecimals = 2, Value = 7.76},
                new Model {ItemName = "Item 3", ItemType = "A", NumberOfDecimals = 2, Value = 2.3670},
                new Model {ItemName = "Item 1", ItemType = "B", NumberOfDecimals = 0, Value = 10},
                new Model {ItemName = "Item 2", ItemType = "B", NumberOfDecimals = 0, Value = 11},
                new Model {ItemName = "Item 3", ItemType = "B", NumberOfDecimals = 0, Value = 3},
                new Model {ItemName = "Item 4", ItemType = "B", NumberOfDecimals = 0, Value = 5}
            };

            ValueList = new ItemsChangeObservableCollection<Model>(valueList);

            double sumA = 0;
            double sumB = 0;
            int indexA = 0;
            int indexB = -1;
            foreach (var item in ValueList)
            {
                if (item.ItemType.Equals("A"))
                {
                    sumA += item.Value;
                }
                if (item.ItemType.Equals("B"))
                {
                    sumB += item.Value;
                    if (indexB == -1)
                    {
                        indexB = ValueList.IndexOf(item);
                    }
                }
            }
            ValueList.Insert(indexA, new Model { ItemName = "Sum A", ItemType = "A", NumberOfDecimals = 2, Value = sumA });
            ValueList.Insert(indexB + 1, new Model { ItemName = "Sum B", ItemType = "B", NumberOfDecimals = 0, Value = sumB });
            ValueList.ItemInCollectionChangeEvent += ValueList_ItemInCollectionChanged;
        }

        private void ValueList_ItemInCollectionChanged(object sender, ItemInCollectionChangeEventArgs e)
        {
            var item = e.Item as Model;
            if (item == null || item.ItemName.StartsWith("Sum"))
            {
                return;
            }

            var columnName = e.PropertyChangedEventArgs.PropertyName;
            if (!columnName.Equals("Value"))
            {
                return;
            }

            // get indexes to calculate partial sum (group by ItemType)
            var itemIndex = ValueList.IndexOf(item);
            var sumIndex1 = ValueList.Where(i => i.ItemName.StartsWith("Sum") && ValueList.IndexOf(i) < itemIndex).Max(ValueList.IndexOf);
            var sumIndex2 = ValueList.Where(i => i.ItemName.StartsWith("Sum") && ValueList.IndexOf(i) > itemIndex).Select(ValueList.IndexOf).DefaultIfEmpty(ValueList.Count).Min();

            // calculate sum
            double sum = 0;
            for (var i = sumIndex1 + 1; i < sumIndex2; i++)
            {
                sum += (double?)ValueList[i][columnName] ?? 0;
            }
            if (Math.Abs(sum) > 0)
            {
                ValueList[sumIndex1][columnName] = sum;
            }
            else
            {
                ValueList[sumIndex1][columnName] = null;
            }

        }

    }
}
